'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extractPersistDirectivePaths = exports.addPersistFieldToDocument = undefined;

var _graphql = require('graphql');

var _apolloUtilities = require('apollo-utilities');

var PERSIST_FIELD = {
  kind: 'Field',
  name: {
    kind: 'Name',
    value: '__persist'
  }
};

var addPersistFieldToSelectionSet = function addPersistFieldToSelectionSet(selectionSet) {
  var isRoot = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  if (selectionSet.selections) {
    if (!isRoot) {
      var alreadyHasThisField = selectionSet.selections.some(function (selection) {
        return selection.kind === 'Field' && selection.name.value === '__typename';
      });

      if (!alreadyHasThisField) {
        selectionSet.selections.push(PERSIST_FIELD);
      }
    }

    selectionSet.selections.forEach(function (selection) {
      // Must not add __typename if we're inside an introspection query
      if (selection.kind === 'Field') {
        if (selection.name.value.lastIndexOf('__', 0) !== 0 && selection.selectionSet) {
          addPersistFieldToSelectionSet(selection.selectionSet);
        }
      } else if (selection.kind === 'InlineFragment') {
        if (selection.selectionSet) {
          addPersistFieldToSelectionSet(selection.selectionSet);
        }
      }
    });
  }
};

var addPersistFieldToDocument = function addPersistFieldToDocument(doc) {
  (0, _apolloUtilities.checkDocument)(doc);
  var docClone = (0, _apolloUtilities.cloneDeep)(doc);

  docClone.definitions.forEach(function (definition) {
    var isRoot = definition.kind === 'OperationDefinition';
    addPersistFieldToSelectionSet(definition.selectionSet, isRoot);
  });

  return docClone;
};

var extractPersistDirectivePaths = function extractPersistDirectivePaths(originalQuery) {
  var directive = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'persist';

  var paths = [];

  var query = (0, _graphql.visit)(originalQuery, {
    Directive: function Directive(_ref, key, parent, path, ancestors) {
      var name = _ref.name.value;

      if (name === directive) {
        paths.push(ancestors.filter(function (_ref2) {
          var kind = _ref2.kind;
          return kind === 'Field';
        }).map(function (_ref3) {
          var name = _ref3.name.value;
          return name;
        }));

        return null;
      }
    }
  });

  return { query: query, paths: paths };
};

exports.addPersistFieldToDocument = addPersistFieldToDocument;
exports.extractPersistDirectivePaths = extractPersistDirectivePaths;